# (c) 2020 Amazon Web Services, Inc. or its affiliates. All Rights Reserved. This AWS Content is provided subject to the terms of the AWS Customer
# Agreement available at  or other written agreement between Customer and Amazon Web Services, Inc.
######
# Checks for compliance on activities that could generate violations
# API Triggers:
# Services: Cloudwatch Events (trigger), dynamodb
######

import os
import json
import boto3
from botocore.exceptions import ClientError

client = boto3.client('dynamodb')
required_keys = os.environ["required_keys"].split(",")
outbound_topic_arn = os.environ["outbound_topic_arn"]


def lambda_handler(event, context):
    print(json.dumps(event))

    # Processes only in the event of Adding or Removing Tags from DynamoDB Table
    if event["detail"]["eventName"] == "TagResource" or event["detail"]["eventName"] == "UntagResource":
        check_tag(event)
    else:
        print("This event doesn't involve modifying tags")

# Retrieve the table arn from the response body of AWS CloudWatch event


def get_table_name(event):
    table_names = []
    table_names.append(event["detail"]["requestParameters"]["resourceArn"])
    return table_names

# Checks the tags for the DynamoDB Table in question


def check_tag(event):
    response = get_table_name(event)
    table_keys = []
    missing_keys = []
    for table_arn in response:
        try:
            describe_tags_response = client.list_tags_of_resource(
                ResourceArn=table_arn)
        except ClientError as err:
            print(err)
            return False
    for table_tag in describe_tags_response["Tags"]:
        if table_tag["Key"] in required_keys:
            table_keys.append(table_tag["Key"])

    # Determine if tag from DynamoDB table exists in the required strings list
    for key in required_keys:
        if key not in table_keys:
            missing_keys.append(key)

    if missing_keys:
        send_violation(event, missing_keys)
    else:
        return None

# Send violation to sns topic


def send_violation(event, missing_keys):
    table = ''.join(get_table_name(event))
    table_arn = table.split("/")
    table_name = table_arn[1]
    missing_keys = ', '.join(missing_keys)
    findsnsregion = outbound_topic_arn.split(":")
    snsregion = findsnsregion[3]
    sendclient = boto3.client('sns', region_name=snsregion)
    try:
        sendclient.publish(
            TopicArn=outbound_topic_arn,
            Message=("The DynamoDB Table " + table_name + " in the region " +
                     snsregion + " is missing the following keys: " + missing_keys),
            Subject="Violation - DynamoDB Table is missing keys!"
        )
    except ClientError as err:
        print(err)
        return False
